package com.zhili.zkservicecenter.component;


import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.zhili.zkservicecenter.configuration.ServerConfig;
import com.zhili.zkservicecenter.dto.*;
import com.zhili.zkservicecenter.entity.ExchangeRecord;
import com.zhili.zkservicecenter.entity.StationInfo;
import com.zhili.zkservicecenter.mapper.DriverVehExchangeBindMapper;;
import com.zhili.zkservicecenter.mapper.VehInfoMapper;
import com.zhili.zkservicecenter.service.IExchangeRecordService;
import com.zhili.zkservicecenter.service.IStationInfoService;
import lombok.extern.slf4j.Slf4j;
import org.apache.curator.framework.CuratorFramework;
import org.apache.ibatis.annotations.Param;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.script.RedisScript;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.concurrent.TimeUnit;

@Component
@Slf4j
public class KafkaMessageHandler {

    @Autowired
    ServerConfig serverConfig;

    @Autowired
    RedisTemplate redisTemplate;

    @Autowired
    RedisHelper redisHelper;

    @Autowired
    ServiceCenter serviceCenter;

    @Autowired
    KafkaTemplate kafkaTemplate;
    @Autowired
    IExchangeRecordService exchangeRecordService;
    @Autowired
    IStationInfoService stationInfoService;

    @Resource(name = "subscribeExchangeScript")
    RedisScript<List> subscribeScript;
    @Autowired
    DriverVehExchangeBindMapper driverVehExchangeBindMapper;
    @Autowired
    VehInfoMapper vehInfoMapper;

    public KafkaMessageHandler(CuratorFramework client) {
    }

    /**
     * 回复启动换电
     */
    @KafkaListener(topics = {"#{serverConfig.getServiceName()}_ExchangeStartReply"}, groupId = "#{serverConfig.getServiceName()}")
    public void processExchangeStartReply(ConsumerRecord<String, String> record, Acknowledgment ack) {
        String value = record.value();
        ExchangeStartReply exchangeEvent = JSON.parseObject(value, ExchangeStartReply.class);
        String exchangeNo = exchangeEvent.getExchangeNo();
        //log.info("回复启动换电：{}" ,exchangeEvent);
        //循环正在监听当前换电号的wx服务器连接
        Set<String> serverSessions = redisHelper.scan("SessionsSubscribeExchange:" + exchangeNo + ":");
        for (String key : serverSessions) {
            String[] split = key.split(":");
            DriverConnInfo driverConnInfo = new DriverConnInfo();
            driverConnInfo.setServerInstanceId(split[2]);
            driverConnInfo.setSessionId(split[3]);
            Boolean exist = serviceCenter.driverServerExist(driverConnInfo.getServerInstanceId());
            if (exist) {
                SessionExchangeEvent sEvent = new SessionExchangeEvent();
                sEvent.setConnInfo(driverConnInfo);
                sEvent.setExchangeStartReply(exchangeEvent);
                log.info("回复启动换电topic：{}" ,driverConnInfo.getServerInstanceId()+"_ExchangeStartReply");
                kafkaTemplate.send(driverConnInfo.getServerInstanceId()+"_ExchangeStartReply", JSON.toJSONString(sEvent));
            }
        }
        ack.acknowledge();
    }

    /**
     * 换电历史实时记录
     */
    @KafkaListener(topics = {"#{serverConfig.getServiceName()}_ExchangeEvent"}, groupId = "#{serverConfig.getServiceName()}")
    public void processExchangeEvent(ConsumerRecord<String, String> record, Acknowledgment ack) {
        String value = record.value();
        ExchangeEvent exchangeEvent = JSON.parseObject(value, ExchangeEvent.class);
        log.info("recv exchangeEvent:" + exchangeEvent);
        String exchangeNo = exchangeEvent.getExchangeNo();
        String stationCode = exchangeEvent.getStationCode();
        //注册站的当前换电号
        //{"StationCurrentExchange_st123":"ex_abcdef"}
        redisTemplate.opsForValue().set("StationCurrentExchange:" + stationCode, exchangeEvent, 12, TimeUnit.HOURS);
        //换电详情缓存成list
        redisTemplate.opsForList().rightPush("ExchangeDetail:" + exchangeNo, exchangeEvent);
        redisTemplate.expire("ExchangeDetail:" + exchangeNo, 12, TimeUnit.HOURS);
        //循环正在监听当前换电号的wx服务器连接
        Set<String> serverSessions = redisHelper.scan("SessionsSubscribeExchange:" + exchangeNo + ":");
        //log.info("serverSessions：" + serverSessions);
        for (String key : serverSessions) {
            String[] split = key.split(":");
            DriverConnInfo driverConnInfo = new DriverConnInfo();
            driverConnInfo.setServerInstanceId(split[2]);
            driverConnInfo.setSessionId(split[3]);
            Boolean exist = serviceCenter.driverServerExist(driverConnInfo.getServerInstanceId());
            if (exist) {
                SessionExchangeEvent sEvent = new SessionExchangeEvent();
                sEvent.setConnInfo(driverConnInfo);
                sEvent.setEvent(exchangeEvent);
                sEvent.setType(1);
                kafkaTemplate.send(driverConnInfo.getServerInstanceId(), JSON.toJSONString(sEvent));
            }
        }
        ack.acknowledge();
    }


    @KafkaListener(topics = {"#{serverConfig.getServiceName()}_ChargerState"}, groupId = "#{serverConfig.getServiceName()}")
    public void processChargerState(ConsumerRecord<String, String> record, Acknowledgment ack) {
        String value = record.value();
        ChargerState chargerState = JSON.parseObject(value, ChargerState.class);
        log.info("recv chargerState:" + chargerState);
        String stationCode = chargerState.getStationCode();
        //架载机信息
        redisTemplate.opsForValue().set("StationChargerState:" + stationCode + "_" + chargerState.getChargerId() + "_" + chargerState.getGunNo(), JSON.toJSONString(chargerState), 12, TimeUnit.HOURS);
        ack.acknowledge();
    }

    /**
     * 订阅换电实时事件记录
     */
    @KafkaListener(topics = {"#{serverConfig.getServiceName()}_subscribeExchangeEvent"}, groupId = "#{serverConfig.getServiceName()}")
    public void driverSubscribeExchangeEvent(ConsumerRecord<String, String> record, Acknowledgment ack) {
        String value = record.value();
        SubscribeExchangeEvent event = JSON.parseObject(value, SubscribeExchangeEvent.class);
        switch (event.getType()) {
            case "on":
//                开启监听
                //log.info("DriverSubscribeExchangeEvent on: {}", event);
                List<String> keys = new ArrayList<>();
                keys.add("ExchangeDetail:" + event.getExchangeNo());
                keys.add("SessionsSubscribeExchange:" + event.getExchangeNo() + ":" + event.getConnInfo().getServerInstanceId() + ":" + event.getConnInfo().getSessionId());
//                采用Lua脚本确定原子性，1.设置监听实时消息 2.读取换电历史详情，预先推送
                List res = (List) redisTemplate.execute(subscribeScript, keys, event.getConnInfo());
                //log.info("res:{}", res);
                List<ExchangeEvent> liseEvent = new ArrayList<>();
                SessionExchangeEvent sEvent = new SessionExchangeEvent();
                ObjectMapper objectMapper = new ObjectMapper();
                for (Object s : res) {
                    ExchangeEvent exchangeEvent = objectMapper.convertValue(s, ExchangeEvent.class);
                    liseEvent.add(exchangeEvent);
                }
                sEvent.setHistory(liseEvent);
                sEvent.setType(0);
                sEvent.setConnInfo(event.getConnInfo());
                SessionExchangeEvent sessionExchangeEvent = objectMapper.convertValue(sEvent, SessionExchangeEvent.class);
                kafkaTemplate.send(event.getConnInfo().getServerInstanceId(), JSON.toJSONString(sessionExchangeEvent));
                break;
            case "off":
//                关闭监听
                redisTemplate.delete("SessionsSubscribeExchange:" + event.getExchangeNo() + ":" + event.getConnInfo().getServerInstanceId() + ":" + event.getConnInfo().getSessionId());
                break;
            default:
                break;
        }
        ack.acknowledge();
    }

    /**
     * 发起换电
     */
    @KafkaListener(topics = {"#{serverConfig.getServiceName()}_stationControlEvent"}, groupId = "#{serverConfig.getServiceName()}")
    public void driverControlStationEvent(ConsumerRecord<String, String> record, Acknowledgment ack) {
        String value = record.value();
        //log.info("driverControlStationEvent:" + value);
        StationControlEvent event = JSON.parseObject(value, StationControlEvent.class);
        switch (event.getType()) {
            case "startExchange":
                //System.out.println("发起换电--------------------------");
                String stationCode = event.getStationCode();
                Object o = redisTemplate.opsForValue().get("StationConn:" + stationCode);
                ObjectMapper objectMapper = new ObjectMapper();
                StationConnInfo stationConnInfo = objectMapper.convertValue(o, StationConnInfo.class);
                if (stationConnInfo == null) {
                    log.info("stationConnInfo({}) not exist", stationCode);
                    break;
                }
                event.setStationConnInfo(stationConnInfo);
                //log.info("发起换电数据{}{}", stationConnInfo.getServerInstanceId(),event);
                kafkaTemplate.send(stationConnInfo.getServerInstanceId(), JSON.toJSONString(event));
                break;
            default:
                break;
        }
        ack.acknowledge();
    }

    /**
     * 上传保存换电记录
     */
    @KafkaListener(topics = {"#{serverConfig.getServiceName()}_UploadExchangeRecord"}, groupId = "#{serverConfig.getServiceName()}")
    public void UploadExchangeRecord(ConsumerRecord<String, String> record, Acknowledgment ack) {
        String value = record.value();
        log.info("上传换电记录数据:" + value);
        ExchangeRecord event = JSON.parseObject(value, ExchangeRecord.class);
        try {
            QueryWrapper<StationInfo> queryWrapper = new QueryWrapper<StationInfo>().eq("station_code",event.getStationCode());
            StationInfo stationInfo = stationInfoService.getOne(queryWrapper);
            if(stationInfo!=null && stationInfo.getAddress()!=null){
                event.setStationAddress(stationInfo.getAddress());
            }
            event.setCreateTime(new Date());
            boolean res = exchangeRecordService.save(event);
            if(res){
                driverVehExchangeBindMapper.saveVehBind(event.getDriverId(),event.getVin(),event.getPlate(),
                        event.getVehicleId(),event.getModel(),event.getTotalMileage(),event.getExchangePower(),event.getId(),new Date());
                vehInfoMapper.updateVeh(event.getVin(),event.getExchangePower(),new Date());
            }
            redisTemplate.delete("StationCurrentExchange:" + event.getStationCode());
        }catch (Exception e){
            log.error("云端保存换电记录失败:{}",value);
        }
        ack.acknowledge();
    }

}
