package com.zhili.zkservicecenter.component;

import com.zhili.zkservicecenter.configuration.ServerConfig;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

@Component
@Slf4j
public class ServiceCenter {
    static Map<String, Map<String, ServerConfig>> servers = new ConcurrentHashMap<>();

    public static void addServer(String role, String id, ServerConfig config) {
        servers.compute(role, (k, v) -> {
            if (v == null) {
                Map<String, ServerConfig> m = new HashMap<>();
                m.put(id, config);
                return m;
            } else {
                v.put(id, config);
                return v;
            }
        });
    }

    public static void remServer(String role, String id) {
        servers.compute(role, (k, v) -> {
            if (v == null) {
                return new HashMap<>();
            } else {
                v.remove(id);
                return v;
            }
        });
    }

    @Scheduled(fixedRate = 10000)
    public void printServers() {
        log.info("monitor--------");
        servers.forEach((k, m) -> {
            log.info("\t" + k + ":");
            m.forEach((id, config) -> {
                log.info("\t\t" + id + ":" + config);
            });
        });
        log.info("--------");
    }

    public Boolean driverServerExist(String serverInstanceId){
        try{
            Map<String, ServerConfig> stationServers = servers.get("DriverServer");
            return stationServers.keySet().contains(serverInstanceId);
        }catch (Exception e){
            e.printStackTrace();
            return false;
        }

    }
}
